#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

#define DBGI(a) printf(#a " ==> %d\n", a)
#define DBGF(a) printf(#a " ==> %f\n", a)
#define DBGC(a) printf(#a " ==> %c\n", a)
#define DBGS(a) printf(#a " ==> %s\n", a)
#define DBGA(a) printf(#a " ==> %x\n", a)
#define HIM printf ("Here I am\n")


double ** alloca_mtx (int n) {
  double ** a, * temp;
  int i;

  if ( (a = (double **) malloc (sizeof (double *) * n)) == NULL) {
    return NULL;
  }
  else {
    if ( (temp = (double *) malloc (sizeof (double) * n * n)) == NULL) {
      return NULL;
    }
    else {
      for (i=0; i<n; i++) 
        a[i] = &temp[i*n]; 
    }
  }

  return a;
} 

void free_mtx (double ** a) {

  free (a[0]);
  free (a);

  return;
}

void fill_mtx (double ** a, int n) {
  int i, j;

  for (i=0; i<n; i++) 
    for (j=0; j<n; j++)
      a[i][j] = 1.0e0 + (double) (1.0e0 * n * (rand() / (RAND_MAX + 1.0e0))); 
}

void print_mtx (double ** a, int n, FILE * fp) {
  int i, j;

  fprintf (fp, "\n");
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++) {
      fprintf (fp, "%10.5f ", a[i][j]);
    }
    fprintf (fp, "\n");
  }

  return;
}

void usage (char * name) 
{

  fprintf (stderr, "usage: %s [options] -n dim [filename]\n\n", name);
  fprintf (stderr, " -n, --dim[=dim]        : int \n");
  fprintf (stderr, " -h, --help             : display this help and exit\n");

  exit (1);
}

int main (int argc, char ** argv) 
{

  int n = -1, usefile;
  double ** a;
  FILE * fp;

  while (1) {
    int c;
    static struct option long_options[] = {
      {"help", no_argument, NULL, 'h'},
      {"dim", required_argument, NULL, 'n'},
      {0,0,0,0}
    };

    c = getopt_long (argc, argv, "hn:", long_options, NULL);
    
    if (c == -1)
      break;

    switch (c) {
      case 'n':
        if ( (n = atoi (optarg)) < 0) {
          fprintf (stderr, "N must be gt 0\n");
          return EXIT_FAILURE;
        }
        break;
      case 'h':
        usage (argv[0]);
      default:
        usage (argv[0]);
        break;
    }
  }

  if (n<0)
    usage (argv[0]);

  // optind variabile globale indice di posizione
  if (optind == argc) 
  {
    usefile = 0;
    fp = stdout;
  }
  else if (optind == argc-1) 
  {
    usefile  = 1;
    if ((fp = fopen (argv[optind], "w")) == NULL) 
    {
      fprintf (stderr, "error in opening file %\n", argv[optind]);
      return EXIT_FAILURE;
    }
  }
  else
    usage (argv[0]);

  if ((a = alloca_mtx (n)) == NULL) 
  {
    fprintf (stderr, "Memory allocation error\n");
    return EXIT_FAILURE;
  }

  fill_mtx (a, n);

  print_mtx (a, n, fp);
  
  free_mtx (a);

  if (usefile)
    fclose (fp);

  return EXIT_SUCCESS;
}
